@@ -4,6 +4,7 @@
 
 #include <linux/input-event-codes.h>
 #include "retro-cairo-display.h"
+#include "retro-gl-display.h"
 #include "retro-controller-codes.h"
 #include "retro-core-view-controller.h"
 #include "retro-pa-player.h"
@@ -31,10 +32,8 @@ struct _RetroCoreView
 {
   GtkEventBox parent_instance;
   RetroCore *core;
-  RetroCairoDisplay *display;
-  GBinding *pixbuf_binding;
+  RetroGLDisplay *display;
   GBinding *sensitive_binding;
-  GdkPixbuf *pixbuf;
   RetroPaPlayer *audio_player;
   gboolean can_grab_pointer;
   gboolean snap_pointer_to_borders;
@@ -56,8 +55,7 @@ struct _RetroCoreView
 G_DEFINE_TYPE (RetroCoreView, retro_core_view, GTK_TYPE_EVENT_BOX)
 
 enum {
-  PROP_PIXBUF = 1,
-  PROP_CAN_GRAB_POINTER,
+  PROP_CAN_GRAB_POINTER = 1,
   PROP_SNAP_POINTER_TO_BORDERS,
   N_PROPS,
 };
@@ -235,6 +233,8 @@ retro_core_view_on_button_press_event (GtkWidget      *source,
   g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
+  gtk_widget_grab_focus (GTK_WIDGET (source));
+
   if (retro_core_view_get_can_grab_pointer (self)) {
     if (retro_core_view_get_is_pointer_grabbed (self))
       set_input_pressed (self->mouse_button_state, event->button);
@@ -247,11 +247,11 @@ retro_core_view_on_button_press_event (GtkWidget      *source,
   else {
     set_input_pressed (self->mouse_button_state, event->button);
     self->pointer_is_on_display =
-      retro_cairo_display_get_coordinates_on_display (self->display,
-                                                      event->x,
-                                                      event->y,
-                                                      &self->pointer_x,
-                                                      &self->pointer_y);
+      retro_gl_display_get_coordinates_on_display (self->display,
+                                                   event->x,
+                                                   event->y,
+                                                   &self->pointer_x,
+                                                   &self->pointer_y);
   }
 
   return FALSE;
@@ -312,11 +312,11 @@ retro_core_view_on_motion_notify_event (GtkWidget      *source,
   }
   else {
     self->pointer_is_on_display =
-      retro_cairo_display_get_coordinates_on_display (self->display,
-                                                      event->x,
-                                                      event->y,
-                                                      &self->pointer_x,
-                                                      &self->pointer_y);
+      retro_gl_display_get_coordinates_on_display (self->display,
+                                                   event->x,
+                                                   event->y,
+                                                   &self->pointer_x,
+                                                   &self->pointer_y);
 
   }
 
@@ -365,9 +365,7 @@ retro_core_view_finalize (GObject *object)
 
   g_clear_object (&self->core);
   g_object_unref (self->display);
-  g_object_unref (self->pixbuf_binding);
   g_object_unref (self->sensitive_binding);
-  g_clear_object (&self->pixbuf);
   g_object_unref (self->audio_player);
   g_hash_table_unref (self->key_state);
   g_hash_table_unref (self->mouse_button_state);
@@ -386,10 +384,6 @@ retro_core_view_get_property (GObject    *object,
   RetroCoreView *self = RETRO_CORE_VIEW (object);
 
   switch (prop_id) {
-  case PROP_PIXBUF:
-    g_value_set_object (value, retro_core_view_get_pixbuf (self));
-
-    break;
   case PROP_CAN_GRAB_POINTER:
     g_value_set_boolean (value, retro_core_view_get_can_grab_pointer (self));
 
@@ -414,10 +408,6 @@ retro_core_view_set_property (GObject      *object,
   RetroCoreView *self = RETRO_CORE_VIEW (object);
 
   switch (prop_id) {
-  case PROP_PIXBUF:
-    retro_core_view_set_pixbuf (self, g_value_get_object (value));
-
-    break;
   case PROP_CAN_GRAB_POINTER:
     retro_core_view_set_can_grab_pointer (self, g_value_get_boolean (value));
 
@@ -442,16 +432,6 @@ retro_core_view_class_init (RetroCoreViewClass *klass)
   object_class->get_property = retro_core_view_get_property;
   object_class->set_property = retro_core_view_set_property;
 
-  properties[PROP_PIXBUF] =
-    g_param_spec_object ("pixbuf",
-                         "Pixbuf",
-                         "The displayed pixbuf",
-                         gdk_pixbuf_get_type (),
-                         G_PARAM_READWRITE |
-                         G_PARAM_STATIC_NAME |
-                         G_PARAM_STATIC_NICK |
-                         G_PARAM_STATIC_BLURB);
-
   properties[PROP_CAN_GRAB_POINTER] =
     g_param_spec_boolean ("can-grab-pointer",
                           "Can grab pointer",
@@ -486,18 +466,14 @@ RetroCoreView* retro_core_view_construct (GType object_type) {
 static void
 retro_core_view_init (RetroCoreView *self)
 {
+  g_object_set ((GtkWidget*) self, "can-default", TRUE, NULL);
   g_object_set ((GtkWidget*) self, "can-focus", TRUE, NULL);
 
-  self->display = g_object_ref_sink (retro_cairo_display_new ());
+  self->display = g_object_ref_sink (retro_gl_display_new ());
   gtk_widget_set_visible (GTK_WIDGET (self->display), TRUE);
   g_object_set (GTK_WIDGET (self->display), "can-focus", FALSE, NULL);
   gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->display));
 
-  self->pixbuf_binding =
-    g_object_bind_property (G_OBJECT (self->display), "pixbuf",
-                            G_OBJECT (self), "pixbuf",
-                            G_BINDING_BIDIRECTIONAL |
-                            G_BINDING_SYNC_CREATE);
   self->sensitive_binding =
     g_object_bind_property (G_OBJECT (self), "sensitive",
                             G_OBJECT (self->display), "sensitive",
@@ -537,13 +513,13 @@ retro_core_view_set_core (RetroCoreView *self,
 
   if (self->core != NULL) {
     g_clear_object (&self->core);
-    retro_cairo_display_set_core (self->display, NULL);
+    retro_gl_display_set_core (self->display, NULL);
     retro_pa_player_set_core (self->audio_player, NULL);
   }
 
   if (core != NULL) {
     self->core = g_object_ref (core);
-    retro_cairo_display_set_core (self->display, core);
+    retro_gl_display_set_core (self->display, core);
     retro_pa_player_set_core (self->audio_player, core);
   }
 }
@@ -562,8 +538,7 @@ retro_core_view_set_pixbuf (RetroCoreView *self,
   g_return_if_fail (RETRO_IS_CORE_VIEW (self));
   g_return_if_fail (pixbuf == NULL || GDK_IS_PIXBUF (pixbuf));
 
-  if (g_set_object (&self->pixbuf, pixbuf))
-    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PIXBUF]);
+  retro_gl_display_set_pixbuf (self->display, pixbuf);
 }
 
 /**
@@ -579,7 +554,7 @@ retro_core_view_get_pixbuf (RetroCoreView *self)
 {
   g_return_val_if_fail (RETRO_IS_CORE_VIEW (self), NULL);
 
-  return self->pixbuf;
+  return retro_gl_display_get_pixbuf (self->display);
 }
 
 /**
@@ -595,7 +570,7 @@ retro_core_view_set_filter (RetroCoreView    *self,
 {
   g_return_if_fail (RETRO_IS_CORE_VIEW (self));
 
-  retro_cairo_display_set_filter (self->display, filter);
+  retro_gl_display_set_filter (self->display, filter);
 }
 
 /**
