@@ -16,6 +16,8 @@ enum {
   RETRO_CORE_ERROR_NO_CALLBACK,
 };
 
+G_DEFINE_QUARK (retro-core-error, retro_core_error)
+
 G_DEFINE_TYPE (RetroCore, retro_core, G_TYPE_OBJECT)
 
 enum {
@@ -61,12 +63,6 @@ static void retro_core_set_game_loaded (RetroCore *self,
 
 /* Private */
 
-static GQuark
-retro_core_error_quark (void)
-{
-  return g_quark_from_static_string ("retro-core-error-quark");
-}
-
 void
 retro_core_push_cb_data (RetroCore *self)
 {
@@ -134,6 +130,7 @@ retro_core_finalize (GObject *object)
   RetroCore *self = RETRO_CORE (object);
   RetroUnloadGame unload_game;
   RetroDeinit deinit;
+  gsize i;
 
   g_return_if_fail (RETRO_IS_CORE (self));
 
@@ -150,6 +147,9 @@ retro_core_finalize (GObject *object)
     g_strfreev (self->media_uris);
 
   g_object_unref (self->module);
+  for (i = 0; i < RETRO_CONTROLLER_TYPE_COUNT; i++)
+    if (self->default_controllers[i] != NULL)
+      g_object_unref (self->default_controllers[i]);
   g_hash_table_unref (self->controllers);
   g_object_unref (self->options);
 
@@ -587,35 +587,6 @@ retro_core_get_name (RetroCore *self)
   return system_info.library_name;
 }
 
-static void
-retro_core_controller_connected (RetroCore       *self,
-                                 guint            port,
-                                 RetroController *controller)
-{
-  RetroControllerType controller_type;
-
-  g_return_if_fail (RETRO_IS_CORE (self));
-  g_return_if_fail (controller != NULL);
-
-  if (!retro_core_get_is_initiated (self))
-    return;
-
-  controller_type = retro_controller_get_controller_type (controller);
-  retro_core_set_controller_port_device (self, port, controller_type);
-}
-
-static void
-retro_core_controller_disconnected (RetroCore *self,
-                                    guint      port)
-{
-  g_return_if_fail (RETRO_IS_CORE (self));
-
-  if (!retro_core_get_is_initiated (self))
-    return;
-
-  retro_core_set_controller_port_device (self, port, RETRO_CONTROLLER_TYPE_NONE);
-}
-
 static void
 retro_core_send_input_key_event (RetroCore                *self,
                                  gboolean                  down,
@@ -1738,9 +1709,7 @@ retro_core_poll_controllers (RetroCore *self)
  * retro_core_get_controller_input_state:
  * @self: a #RetroCore
  * @port: the port number
- * @controller_type: a #RetroControllerType to query @self
- * @index: an input index to interpret depending on @controller_type
- * @id: an input id to interpret depending on @controller_type
+ * @input: a #RetroInput
  *
  * Gets the state of an input of the controller plugged into the given port of
  * @self.
@@ -1748,31 +1717,29 @@ retro_core_poll_controllers (RetroCore *self)
  * Returns: the input's state
  */
 gint16
-retro_core_get_controller_input_state (RetroCore           *self,
-                                       guint                port,
-                                       RetroControllerType  controller_type,
-                                       guint                index,
-                                       guint                id)
+retro_core_get_controller_input_state (RetroCore  *self,
+                                       guint       port,
+                                       RetroInput *input)
 {
   RetroController *controller;
+  RetroControllerType controller_type;
 
   g_return_val_if_fail (RETRO_IS_CORE (self), 0);
 
-  if (!g_hash_table_contains (self->controllers, &port))
-    return 0;
+  controller_type = retro_input_get_controller_type (input) &
+                    RETRO_CONTROLLER_TYPE_TYPE_MASK;
 
   controller = g_hash_table_lookup (self->controllers, &port);
+  if (controller != NULL &&
+      retro_controller_has_capability (controller, controller_type))
+    return retro_controller_get_input_state (controller, input);
 
-  if (controller == NULL)
-    return 0;
+  controller = self->default_controllers[controller_type];
+  if (controller != NULL &&
+      retro_controller_has_capability (controller, controller_type))
+    return retro_controller_get_input_state (controller, input);
 
-  if ((retro_controller_get_capabilities (controller) & (1 << controller_type)) == 0)
-    return 0;
-
-  return retro_controller_get_input_state (controller,
-                                           controller_type,
-                                           index,
-                                           id);
+  return 0;
 }
 
 // FIXME documentation
@@ -1797,6 +1764,27 @@ retro_core_get_controller_capabilities (RetroCore *self)
   return 0;
 }
 
+/**
+ * retro_core_set_default_controller:
+ * @self: a #RetroCore
+ * @controller_type: a #RetroControllerType
+ * @controller: (nullable): a #RetroController
+ *
+ * Uses @controller as the default controller for the given type. When a port
+ * has no controller plugged plugged into it, the core will use the default
+ * controllers instead.
+ */
+void
+retro_core_set_default_controller (RetroCore           *self,
+                                   RetroControllerType  controller_type,
+                                   RetroController     *controller)
+{
+  g_return_if_fail (RETRO_IS_CORE (self));
+  g_return_if_fail (controller_type < RETRO_CONTROLLER_TYPE_COUNT);
+
+  g_set_object (&self->default_controllers[controller_type], controller);
+}
+
 /**
  * retro_core_set_controller:
  * @self: a #RetroCore
@@ -1811,16 +1799,27 @@ retro_core_set_controller (RetroCore       *self,
                            RetroController *controller)
 {
   guint *port_copy;
+  RetroControllerType controller_type;
 
   g_return_if_fail (RETRO_IS_CORE (self));
-  g_return_if_fail (RETRO_IS_CONTROLLER (controller));
 
-  port_copy = g_new (guint, 1);
-  *port_copy = port;
-  g_hash_table_insert (self->controllers,
-                       port_copy,
-                       g_object_ref (controller));
-  retro_core_controller_connected (self, port, controller);
+  if (RETRO_IS_CONTROLLER (controller)) {
+    port_copy = g_new (guint, 1);
+    *port_copy = port;
+    g_hash_table_insert (self->controllers,
+                         port_copy,
+                         g_object_ref (controller));
+    controller_type = retro_controller_get_controller_type (controller);
+  }
+  else {
+    g_hash_table_remove (self->controllers, &port);
+    controller_type = RETRO_CONTROLLER_TYPE_NONE;
+  }
+
+  if (!retro_core_get_is_initiated (self))
+    return;
+
+  retro_core_set_controller_port_device (self, port, controller_type);
 }
 
 /**
@@ -1859,25 +1858,6 @@ retro_core_set_keyboard (RetroCore *self,
   }
 }
 
-/**
- * retro_core_remove_controller:
- * @self: a #RetroCore
- * @port: the port number
- *
- * Removes the controller plugged into @self at port @port, if any.
- */
-void
-retro_core_remove_controller (RetroCore *self,
-                              guint      port)
-{
-
-  g_return_if_fail (RETRO_IS_CORE (self));
-
-  // FIXME Do that only if a controller is plugged into that port.
-  g_hash_table_remove (self->controllers, &port);
-  retro_core_controller_disconnected (self, port);
-}
-
 /**
  * retro_core_iterate_controllers:
  * @self: a #RetroCore
