@@ -2,7 +2,9 @@
 
 #include "retro-core-private.h"
 
+#include <stdbool.h>
 #include "libretro-environment.h"
+#include "retro-input-private.h"
 #include "retro-pixdata-private.h"
 
 void retro_core_set_system_av_info (RetroCore         *self,
@@ -196,9 +198,10 @@ get_variable (RetroCore     *self,
   return TRUE;
 }
 
+// The data must be bool, not gboolean, the sizes can be different.
 static gboolean
 get_variable_update (RetroCore *self,
-                     gboolean  *update)
+                     bool      *update)
 {
   *update = retro_options_get_variable_update (self->options);
 
@@ -342,7 +345,7 @@ environment_core_command (RetroCore *self,
     return get_variable (self, (RetroVariable *) data);
 
   case RETRO_ENVIRONMENT_GET_VARIABLE_UPDATE:
-    return get_variable_update (self, (gboolean *) data);
+    return get_variable_update (self, (bool *) data);
 
   case RETRO_ENVIRONMENT_SET_DISK_CONTROL_INTERFACE:
     set_disk_control_interface (self, (RetroDiskControlCallback *) data);
@@ -500,13 +503,16 @@ on_input_state (guint port,
                 guint id)
 {
   RetroCore *self;
+  RetroInput input;
 
   self = retro_core_get_cb_data ();
 
   if (self == NULL)
     g_return_val_if_reached (0);
 
-  return retro_core_get_controller_input_state (self, port, device, index, id);
+  retro_input_init (&input, device, id, index);
+
+  return retro_core_get_controller_input_state (self, port, &input);
 }
 
 // TODO This is internal, make it private as soon as possible.
