@@ -3,6 +3,10 @@
 #ifndef RETRO_ROTATION_H
 #define RETRO_ROTATION_H
 
+#if !defined(__RETRO_GTK_INSIDE__) && !defined(RETRO_GTK_COMPILATION)
+# error "Only <retro-gtk.h> can be included directly."
+#endif
+
 #include <glib.h>
 
 G_BEGIN_DECLS
